/**************************************************\
*                   S D S E L                      * 
*           (c) 2015 - Daniel Coulom               *  
*           http://dcmoto.free.fr/                 *
*           http://forum.system-cfg.com/           *
*--------------------------------------------------*
* Ce code est distribue gratuitement dans l'espoir *
* qu'il sera utile, mais sans aucune  garantie  et *
* sans  engager  la  responsabilit  de  l'auteur. *
* Vous  pouvez  l' utiliser,  le  modifier  et  le *
* diffuser librement, en conservant cette  licence *
* et les rfrences de l'auteur dans   toutes  les *
* copies. L'exploitation commerciale est interdite.*
\**************************************************/

* Ce programme permet de selectionner un fichier .sd 
* dans une liste deroulante et de monter l'image.
* Cette version est compatible avec les cartes SD
* et SDHC sur MO et sur TO
* Restrictions :
* - 364 fichiers maxi dans le repertoire principal
*   en version TO, 273 en version MO 
* - En FAT32 limitation supplementaire fonction
*   de la taille de l'unite d'allocation
*   et de la taille des noms longs :
*   4096 = environ 32 fichiers de 39 caracteres  
*   8192 = environ 64 fichiers de 39 caracteres
*   16Ko = environ 128 fichiers de 39 caracteres
*   32Ko = environ 256 fichiers de 39 caracteres
    
/**************************************************\
*                Version 2014.01.17                *
\**************************************************/
* Historique
* 2015.01.17 amelioration affichage (mode page)
* 2014.12.29 correction erreur calcul LBA
* 2014.12.27 compatibilite MO et TO
* 2014.12.26 calcul du LBA et changement d'image
* 2014.12.19 affichage des noms longs en FAT16
* 2014.12.09 debut du developpement

*------------------------------------------------------
* DEFINITION DES EQUIVALENCES
*------------------------------------------------------
BUFADD  EQU $4F       adresse du buffer du secteur
SD_LB0  EQU $8E       adresse du LBA du fichier .sd 
SD_TYP  EQU $92       type de carte SD=0 SDHC=1
SD_LBA  EQU $94       adresse du LBA du secteur a lire

*------------------------------------------------------
* INITIALISATIONS EN FONCTION DE L'ORDINATEUR
*------------------------------------------------------
  ORG   $9A00 
INIT
  PSHS  U,Y,X,DP,B,A,CC                  
  ORCC  #$50          desactive les interruptions 
  LDA   #$60          valeur initialisation DP pour TO
  LDX   #$1F40        adresse pour test RAM ou ROM
  LDB   ,X            lecture adresse X
  COM   ,X            tente de modifier adresse X
  CMPB  ,X            test modification adresse X
  BEQ   INIT1         pas de difference -> TO
  COM   ,X            retablissement adresse X
  LDD   #$A02B        adresse fonction RS512
  STD   RS512         stockage adresse
  LDD   #$6A00        adresse de la table des noms
  STD   TDEB          adresse debut de table
  STD   TFIN          adresse fin de table
  STD   TTOP          adresse nom en haut de l'ecran
  STD   TSEL          adresse nom selectionne
  LDD   #$9A00        adresse table des noms + $3000
  STD   TMAX          adresse haute de la table des noms
  LDD   #$3F82        affichage caractere MO = SWI #$82
  STD   WCHAR         modification routine affichage
  LDB   #$8A          lecture clavier MO = SWI #$8A
  STD   RCHAR         modification routine lecture
  LDB   #$7B          code inversion video pour MO
  STB   INVER1+1      modification code inversion video
  LDA   #$20          valeur initialisation DP pour MO
INIT1
  TFR   A,DP          DP = $20(MO) ou $60(TO)
  LBSR  LINIT         raz ligne 

*------------------------------------------------------
* LECTURE DU PREMIER SECTEUR DE LA CARTE (MBR ou BOOT)
* Table des partitions en $1BE-$1FD
* Premiere partition en $1BE-$1CD
* - $1BE = Boot Indicator ($00=inactif, $80=actif)
* - $1BF-$1C1 = Starting CHS values
* - $1C2 = Partition type descriptor
*          04: FAT16 size less than 32 MB
*          06: FAT16 size greater or equal 32 MB
*          0B: FAT32
* - $1C3-$1C5 = Ending CHS values
* - $1C6-$1C9 = Starting sector
* - $1CA-$1CD = Partition size in sectors
* Signature en $1FE-$1FF = $55AA
*------------------------------------------------------
  LDD   #BUFFER       adresse du buffer
  STD   <BUFADD       stockage adresse buffer 
  LDD   #$0000        numero du secteur a lire
  STD   <SD_LBA       adresse physique (poids fort)
  STD   <SD_LBA+2     adresse physique (poids faible)
  JSR   [RS512]       lecture secteur 512 octets
  LDD   #$55AA        Signature secteur
  CMPD  BUFFER+$1FE   Test signature
  LBNE  RETOUR        Signature incorrecte
* astuce pour differencier un MBR d'un Boot Sector
* le MBR a toujours $80 ou $00 en $1BE (pas certain)   
  LSL   BUFFER+$1BE   Boot indicator 
  BNE   ABOOT1        Pas de MBR, donc boot record
* seule la premiere partition est exploree,
* son numero de secteur doit tenir dans deux octets
  LDA   BUFFER+$1C7   LBA partition pds fort    
  LDB   BUFFER+$1C6   LBA partition pds faible  
  STD   PSECT         stockage secteur debut partition
  
*------------------------------------------------------
* Lecture du Boot Record
* Premier secteur au debut de la partition
* Registre D = numero secteur de debut de partition
*------------------------------------------------------
RBOOT
  STD   <SD_LBA+2     stockage pour CMD17 si carte SDHC
  TST   <SD_TYP       test du type de carte
  BNE   RBOOT1        carte SDHC, sinon calcul pour SD
  ASLB                multiplie par 2 (poids faible)  
  ROLA                multiplie par 2 (poids fort)
  STD   <SD_LBA+1     adresse = numero secteur * $200  
  CLRB                raz registre A
  STB   <SD_LBA+3     raz dernier octet adresse
RBOOT1 
  JSR   [RS512]       lecture du secteur de 512 octets

*------------------------------------------------------
* Analyse du Boot Sector
* $1FE-$1FF: $55AA (signature)
* $00B-$00C: nombre d'octets par secteur (512)
* $00D: nombre de secteurs par cluster
* $00E-$00F: nombre de secteurs reserves
* $010: nombre de FATs
* $011-$012: nombre entrees directory (FAT16 seulement)
* $016-$017: nombre secteurs FAT (FAT16 seulement)
* $024-$027: nombre secteurs FAT (FAT32 seulement)
* $02C-$02D: numero cluster directory (FAT32)
* Attention: le debut des donnees commence au cluster 2
* - En FAT16: apres la directory
* - En FAT32: apres les FATs
* Pour faciliter les calculs, on retranche 2 fois le
* nombre de secteurs par cluster au debut des donnees.
* 
* Calcul du secteur physique du repertoire principal
* = numero du secteur de debut de partition
* + nombre de secteurs reserves
* + nombre de secteurs FAT * nombre de FAT
*
* Calcul du secteur physique du cluster zero
* = numero du secteur du debut du repertoire principal
* + nombre d'entrees repertoire / 16 (0 en FAT32)
* - nombre de secteurs par cluster * 2
*
*------------------------------------------------------
ABOOT
  LDD   #$55AA        Signature secteur
  CMPD  BUFFER+$1FE   Test signature
  LBNE  RETOUR        Signature incorrecte
ABOOT1
  CLRA                
  LDB   BUFFER+$0D    Nombre de secteurs par cluster
  STD   CSIZ2         Stockage nbre secteurs par cluster
  LDA   BUFFER+$0F    Blocs reserves pds fort   ex: $00
  LDB   BUFFER+$0E    Blocs reserves pds faible ex: $04
  STD   RSECT         stockage nbre secteurs reserves
* taille FAT (en FAT16) dans D  
  LDA   BUFFER+$17    Fat16 size pds fort       ex: $00
  LDB   BUFFER+$16    Fat16 size pds faible     ex: $F2
  STD   BUFFER+$16    stockage taille FAT16
  BNE   ABOOT2        FAT16 si different de zero  
* taille FAT (en FAT32) dans D  
* 2 octets poids fort ignores (simplification abusive ?)
  LDA   BUFFER+$25    Fat32 size pds fort       ex: $03
  LDB   BUFFER+$24    Fat32 size pds faible     ex: $AB
* taille totale fonction du nombre de FATs
ABOOT2
  DEC   BUFFER+$10    Nombre de FAT             ex: $02
  BEQ   ABOOT3        Une seule FAT
  ASLB                FAT size multiplie par 2  ex: $E4 
  ROLA                FAT size multiplie par 2  ex: $01
* Le registre D contient le nombre de secteurs des FATs
* Les calculs suivants sont faux si le premier numero de
* secteur des donnees ne tient pas dans deux octets 
ABOOT3   
  ADDD  RSECT         ajout nombre de secteurs reserves
  ADDD  PSECT         ajout secteur de debut de partition
  STD   DSECT         stockage secteur debut directory
  LDA   BUFFER+$12    nbre entrees directory (poids fort)
  LDB   BUFFER+$11    nbre entrees directory (poids faible)
  LSRA                division par 16
  RORB                pour obtenir
  LSRA                le nombre de secteurs
  RORB                de la directory
  LSRA                En FAT32 le nombre d'entrees  
  RORB                de la directory est nul
  LSRA                Le debut des donnees (cluster 2)
  RORB                commence apres les FATs 
  ADDD  DSECT         ajout secteur debut directory
  SUBD  CSIZ2         retrancher le nbre secteurs/cluster 
  SUBD  CSIZ2         retrancher le nbre secteurs/cluster 
  STD   FSECT         stockage numero secteur du cluster 0
  LDD   DSECT         secteur de debut de directory

*------------------------------------------------------
* LECTURE DE LA DIRECTORY
* D contient le numero du secteur a lire
*------------------------------------------------------
RDSECT
  STD   <SD_LBA+2     stockage pour la commande 17
  TST   <SD_TYP       type de carte 0=SD 1=SDHC
  BNE   RDSEC1        carte SDHC
  ASLB                LBA multiplie par 2 
  ROLA                LBA multiplie par 2
  STD   <SD_LBA+1     LBA multiplie par $100
  CLRB
  STB   <SD_LBA+3     octet de poids faible a zero
RDSEC1
  JSR   [RS512]       Lecture du secteur 
  LDX   #BUFFER       Adresse du buffer

*------------------------------------------------------
* LECTURE DES ENTREES DE LA DIRECTORY
* Le registre X pointe sur l'entree en cours 
*------------------------------------------------------
RDIR1  
  LDB   ,X            lecture premier octet
  LBEQ  AFFIC         $00 = fin de repertoire
  CMPB  #$E5          $E5 = entree annulee
  BEQ   RDIR9         entree suivante
  LDB   11,X          lecture attribut
  ANDB  #$0F          selection demi-octet droit 
  CMPB  #$0F          attribut #$0F ?
  BNE   RDIR6         entree pour nom court
  LBSR  LNAME         entree pour nom long
  BRA   RDIR9         entree suivante       
RDIR6
  LBSR  GETLBA        recherche du LBA
  LDB   [TFIN]        premier caractere du nom
  CMPB  #$20          test espace
  BNE   RDIR7         non : il y a un nom long
  LBSR  SNAME         oui : decodage nom court 
RDIR7
  LDY   TFIN          adresse ligne courante
  LEAY  45,Y          adresse ligne suivante
  CMPY  TMAX          test taille de la table
  BHI   AFFIC         table pleine
  STY   TFIN          stockage adresse
  LBSR  LINIT         raz ligne suivante
RDIR9
  LEAX  32,X          Adresse entree suivante
  CMPX  #BUFFER+$200  Fin du buffer
  BNE   RDIR1         Traitement entree suivante
* lecture secteur suivant
  LDD   DSECT         secteur debut directory 
  ADDD  #$0001        secteur suivant  
  STD   DSECT         nouveau debut directory 
  BRA   RDSECT        lecture secteur directory

*------------------------------------------------------
* AFFICHAGE D'UNE LISTE DE 16 NOMS
*------------------------------------------------------
AFFIC
  LBSR  LOCAT6        positionnement ligne 6
  LDB   #$10          compteur de boucles
  PSHS  B             empilage compteur
  LDY   TTOP          adresse premier nom
AFFIC1
  CMPY  TFIN          test fin de table
  BLS   AFFIC2        fin de table non depassee
  LEAY  -45,Y         rester en fin de table
AFFIC2
  LBSR  DISP          affichage d'une ligne 
  DEC   ,S            decrementation compteur
  BNE   AFFIC1        ligne suivante
  PULS  B             depilage compteur   

*------------------------------------------------------
* TEST DU CLAVIER
*------------------------------------------------------
CLAV
  BSR   RCHAR         lecture du clavier
  CMPB  #$0D          ENTREE
  BEQ   SETLBA        changement image disquette
  CMPB  #$0A          fleche bas
  BEQ   BAS           ligne suivante 
  CMPB  #$0B          fleche haut
  BEQ   HAUT          ligne precedente 
  BRA   CLAV          nouveau test

*------------------------------------------------------
* TRAITEMENT FLECHE VERS LE BAS
*------------------------------------------------------
BAS  
  LDY   TSEL          adresse ligne selectionnee
  LEAY  45,Y          adresse ligne suivante
  CMPY  TFIN          test fin de table 
  BEQ   CLAV          ne rien faire en fin de table
  STY   TSEL          stockage nouvelle adresse
* test de depassement bas ecran
  LDY   TTOP          adresse haut ecran
  LEAY  720,Y         adresse haut ecran suivant
  CMPY  TSEL          comparaison ligne selectionnee 
  BNE   BAS1          pas de changement d'ecran
* changement d'ecran
  STY   TTOP          haut de l'ecran suivant
  BRA   AFFIC         nouvel affichage ecran complet 
* pas de changement d'ecran
BAS1  
  LDY   TSEL          adresse ligne selectionnee
  LEAY  -45,Y         adresse ligne precedente
  LBSR  LOCATY        positionnement sur la ligne
  LBSR  DISP          affichage ligne precedente
  LBSR  DISP          affichage ligne selectionnee
  BRA   CLAV          retour au test du clavier 

*------------------------------------------------------
* TRAITEMENT FLECHE VERS LE HAUT
*------------------------------------------------------
HAUT  
  LDY   TSEL          adresse ligne selectionnee
  CMPY  TDEB          test debut de table 
  BEQ   CLAV          ne rien faire en debut de table
  LEAY  -45,Y         adresse ligne precedente
  STY   TSEL          stockage nouvelle adresse
* test de depassement haut ecran
  LDY   TTOP          adresse haut ecran
  LEAY  -45,Y         adresse ligne precedente
  CMPY  TSEL          comparaison ligne selectionnee 
  BNE   HAUT1         pas de changement d'ecran
* changement d'ecran
  LEAY  -675,Y        adresse page precedente
  STY   TTOP          haut de l'ecran precedent
  LBRA  AFFIC         nouvel affichage ecran complet 
* pas de changement d'ecran
HAUT1  
  LDY   TSEL          adresse ligne selectionnee
  LBSR  LOCATY        positionnement sur la ligne
  LBSR  DISP          affichage ligne selectionnee
  LBSR  DISP          affichage ligne suivante
  BRA   CLAV          retour au test du clavier 

*------------------------------------------------------
* LECTURE DU CLAVIER (RETOUR DANS LE REGISTRE B)
* Version pour TO remplacee par SWI #$8A pour MO
*------------------------------------------------------
RCHAR
  JMP   $E806         lecture du clavier

*------------------------------------------------------
* INITIALISATION LBA POUR CHANGEMENT IMAGE DISQUETTE
*------------------------------------------------------
SETLBA
  LDY   TSEL          adresse ligne selectionnee
  LDA   CSIZE         nombre de secteurs par cluster
* multiplication par le nombre de secteurs/cluster  
SETLB1
  LSRA                divise par 2 
  BEQ   SETLB2
  LSL   44,Y          decalage octet 4
  ROL   43,Y          decalage octet 3
  ROL   42,Y          decalage octet 2
  ROL   41,Y          decalage octet 1
  BRA   SETLB1        nouvelle boucle
* ajout du secteur du cluster zero 
SETLB2
  LDD   FSECT         numero secteur cluster zero
  ADDB  44,Y          ajout octet 4 
  ADCA  43,Y          ajout octet 3 + retenue
  STD   43,Y          stockage octets 3 et 4
  LDD   41,Y          octets 1 et 2
  ADCB  #0            ajout retenue 
  ADCA  #0            ajout retenue
  STD   41,Y          stockage octets 1 et 2
  TST   <SD_TYP       test du type de carte
  BNE   SETLB4        carte SDHC, sinon calcul pour SD
* initialisation LBA du fichier pour carte SD
SETLB3
  CLRB                zero  
  STB   <SD_LB0+3     octet 4 a zero
  LDB   42,Y          octet de poids fort
  LSLB                multiplie par 2
  STB   <SD_LB0       octet 1 (sans retenue)
  LDD   43,Y          octets 2 et 3
  LSLB                multiplication par 2
  ROLA                multiplication par 2
  STD   <SD_LB0+1     stockage octets 2 et 3
  BCC   RETOUR        pas de retenue
  INC   <SD_LB0       ajout retenue
  BRA   RETOUR        retour au Basic
* initialisation LBA du fichier pour carte SDHC
SETLB4
  LDD   41,Y          deux octets de poids fort
  STD   <SD_LB0       stockage
  LDD   43,Y          deux octets de poids faible
  STD   <SD_LB0+2     stockage

*------------------------------------------------------
* RETOUR AU BASIC
*------------------------------------------------------
RETOUR
  PULS  CC,A,B,DP,X,Y,U,PC

*------------------------------------------------------
* LECTURE ET STOCKAGE DU CLUSTER DU FICHIER
*------------------------------------------------------
GETLBA
  LDY   TFIN          Y pointe sur la zone du nom
  LDA   21,X          cluster octet 1
  STA   41,Y          LBA octet 1
  LDA   20,X          cluster octet 2
  STA   42,Y          LBA octet 2
  LDA   27,X          cluster octet 3
  STA   43,Y          LBA octet 3
  LDA   26,X          cluster octet 4
  STA   44,Y          LBA octet 4
  RTS

*------------------------------------------------------
* EXTRACTION DES CARACTERES DU NOM COURT
*------------------------------------------------------
SNAME
  LDY   TFIN          Y pointe sur la zone du nom
  CLRB                compteur de caracteres
SNAME1  
  LDA   B,X           lecture caractere
  CMPA  #$20          test espace
  BEQ   SNAME3        ignorer les espaces
  CMPA  #$41          caractere A  
  BLO   SNAME2        copier si < A 
  CMPA  #$5A          caractere Z
  BHI   SNAME2        copier si > Z
  ORA   #$20          passer en minuscules 
SNAME2  
  STA   ,Y+           stockage caractere
SNAME3
  INCB                caractere suivant
  CMPB  #$08          huitieme caractere
  BNE   SNAME4        sinon rien de plus
  LDA   #$2E          ajouter un point
  STA   ,Y+           stockage point
SNAME4  
  CMPB  #$0B          onzieme caractere
  BNE   SNAME1        caractere suivant
  RTS

*------------------------------------------------------
* EXTRACTION DES CARACTERES DU NOM LONG
*------------------------------------------------------
LNAME
  LDY   TFIN          Y pointe sur la zone du nom
  LDA   #$0D          nbre de caracteres par entree
  LDB   ,X            sequence number
  ANDB  #$BF          slot number
  CMPB  #$03          comparaison avec 3
  BHI   LNAME9        Si superieur ne pas traiter
  MUL                 multiplier par 13
  SUBD  #$0D          moins 13 --> position 1er caractere
  LEAY  B,Y           Y pointe sur 1er caractere
  LDA   1,X           1er caractere
  STA   ,Y+           stockage
  LDA   3,X           2eme caractere
  STA   ,Y+           stockage
  LDA   5,X           3eme caractere
  STA   ,Y+           stockage
  LDA   7,X           4eme caractere
  STA   ,Y+           stockage
  LDA   9,X           5eme caractere
  STA   ,Y+           stockage
  LDA   14,X          6eme caractere
  STA   ,Y+           stockage
  LDA   16,X          7eme caractere
  STA   ,Y+           stockage
  LDA   18,X          8eme caractere
  STA   ,Y+           stockage
  LDA   20,X          9eme caractere
  STA   ,Y+           stockage
  LDA   22,X          10eme caractere
  STA   ,Y+           stockage
  LDA   24,X          11eme caractere
  STA   ,Y+           stockage
  LDA   28,X          12eme caractere
  STA   ,Y+           stockage
  LDA   30,X          13eme caractere
  STA   ,Y+           stockage
  LDB   #$0D          compteur pour 13 boucles
LNAME2
  LDA   ,-Y           caractere du nom
  BEQ   LNAME3        test $00
  INCA                transforme $FF en $00
  BNE   LNAME4        test $FF
LNAME3
  LDA   #$20          blanc remplace $00 et $FF        
  STA   ,Y            stockage dans le nom
LNAME4  
  DECB                decrementation compteur
  BNE   LNAME2        boucle suivante 
LNAME9  
  RTS

*------------------------------------------------------
* AFFICHAGE D'UNE LIGNE POINTEE PAR Y
* AU RETOUR Y POINTE SUR LA LIGNE SUIVANTE
*------------------------------------------------------
DISP
  CMPY  TSEL          test ligne selectionnee
  BNE   DISP1         ligne normale 
  BSR   INVERS        inversion video
DISP1
  LDA   #$29          nombre de caracteres 41
DISP2
  LDB   ,Y+           caractere a ecrire
  BSR   WCHAR         envoi caractere
  DECA                decrementation compteur
  BNE   DISP2         caractere suivant
  LEAY  -41,Y 
  CMPY  TSEL          test ligne selectionnee
  BNE   DISP9         ligne normale 
  BSR   INVERS        retablir video normale
DISP9  
  LEAY  45,Y          adresse ligne suivante
  RTS

*------------------------------------------------------
* INVERSION VIDEO
*------------------------------------------------------
INVERS
  LDB   #$1B          sequence d'echappement
  BSR   WCHAR         envoi caractere
INVER1
  LDB   #$5C          inverse video pour TO
*                     remplace par #$7B pour MO
  BSR   WCHAR         envoi caractere
  RTS

*------------------------------------------------------
* POSITIONNEMENT SUR LA LIGNE 6
*------------------------------------------------------
LOCAT6
  LDB   #$1F          code de positionnement
  BSR   WCHAR         envoi caractere
  LDB   #$46          ligne 6
  BSR   WCHAR         envoi caractere
  LDB   #$41          colonne 1
  BSR   WCHAR         envoi caractere
  RTS

*------------------------------------------------------
* POSITIONNEMENT SUR LA LIGNE POINTEE PAR Y
*------------------------------------------------------
LOCATY
  PSHS  Y             sauvegarde Y
  LDB   #$1F          code de positionnement
  BSR   WCHAR         envoi caractere
  LDB   #$46          code pour pointer ligne 6 
LOC1  
  CMPY  TTOP          comparaison a la premiere ligne
  BEQ   LOC2          positionnement ligne B
  INCB                numero ligne suivante
  LEAY  -45,Y         adresse ligne precedente
  BRA   LOC1          nouvelle boucle
LOC2
  BSR   WCHAR         envoi caractere
  LDB   #$41          colonne 1
  BSR   WCHAR         envoi caractere
  PULS  Y,PC          retour

*------------------------------------------------------
* AFFICHAGE CARACTERE (CARACTERE DANS REGISTRE B)
* Version TO remplacee par SWI #$82 pour MO
*------------------------------------------------------
WCHAR
  JMP   $E803         envoi caractere

*------------------------------------------------------
* INITIALISATION NOUVELLE LIGNE DE 45 CARACTERES
* NOM(39 OCTETS) + CR + LF + ADRESSE(4 OCTETS)
*------------------------------------------------------
LINIT
  LDY   TFIN
  LDD   #$202D        espace et 45 boucles
LINIT1  
  STA   ,Y+           octet initialise a espace
  DECB                decrementation compteur
  BNE   LINIT1        octet suivant
  LDD   #$0D0A        CR+LF
  STD   -6,Y          ajout CR+LF
  RTS

*------------------------------------------------------
* Zones de TRAVAIL
*------------------------------------------------------
RS512
  FDB   $E02B         fonction de lecture secteur
CSIZ2
  FCB   $00           nombre de secteurs par cluster (sur 2 octets)
CSIZE
  FCB   $00           nombre de secteurs par cluster (sur 1 octet)
RSECT
  FDB   $0000         nombre de secteurs reserves
PSECT
  FDB   $0000         numero secteur debut partition
DSECT
  FDB   $0000         numero secteur debut directory
FSECT
  FDB   $0000         numero secteur de data cluster 0    
TDEB
  FDB   $A000         adresse debut table des noms
TFIN
  FDB   $A000         adresse fin table des noms
TTOP
  FDB   $A000         adresse nom en haut d'ecran
TSEL
  FDB   $A000         adresse nom selectionne
TMAX
  FDB   $E000         limite haute de la table des noms
BUFFER  

  END
